#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup() {
	
	// load the default ofxOpenNI config settings
	// see bin/data/openni/config/ofxopenni_config.xml
	context.setupUsingXMLFile();
	
	// uncomment this to see what open ni is doing inside
	context.enableLogging(OF_LOG_VERBOSE);
	
	// setup resources
	depth.setup(&context);
	image.setup(&context);
	user.setup(&context, &depth);
	context.enableLogging(OF_LOG_WARNING);
	
	
	//learned how to use sound from the example of http://www.benmcchesney.com/blog/2010/12/open-frameworks-kinect-sound/
	
	//soundSamples
	booLaugh.loadSound("boolaugh.wav");
	booLaugh.setVolume(0.75f);
	booLaugh.setMultiPlay(false);
	
	ghostHouseMusic.loadSound("Super Mario World Music - Haunted House.mp3");
	ghostHouseMusic.setVolume(0.15f);
	ghostHouseMusic.setLoop(true);
	
	ghostHouseMusic.play();
	
	
	boo.loadImage("boo.png");
	booFlip.loadImage("booFlip.png");
	booBack.loadImage("booBack.png");
	booBackFlip.loadImage("booBackFlip.png");
	
	angle = 0.0;
	
}

//--------------------------------------------------------------
void testApp::update(){
	ofBackground(100, 100, 100);
	
	// update the open ni context and user tracker
	context.update();
	user.update();
}

//--------------------------------------------------------------
void testApp::draw(){
	
	// draw depth and rgb images
	//	image.draw(0,0,ofGetWidth(), ofGetHeight());
	image.draw();
	depth.draw(645, 10, 400, 300);
	
	// draw all the users automatically?
	//	user.draw();
	
	// begin JKH edit
	const unsigned char* cameraPixels = image.image_pixels;
	const XnDepthPixel* theDepth = depth.getDepthPixels();
	const unsigned char* booPixels;
	
	angle += 0.1;
	
	stringstream info;
	info << "number of tracked users: " << user.getNumTrackedUsers() << endl;
	
	
	
	
	// draw the neck positions of each user
	for(int i = 0; i < user.getNumTrackedUsers(); ++i) {
		ofxTrackedUser* tracked = user.getTrackedUser(i);
		if(tracked != NULL) {
			
			ofxLimb neck = tracked->neck;
			if ((&neck != NULL) && (neck.found)) {
				
				info << i << " neck pos: "
				<< tracked->neck.begin.x << " " 
				<< tracked->neck.begin.y << " "
				<< tracked->neck.begin.z << endl; 
				
				ofSetColor(255,255,255);
				float neckx = tracked->neck.end.x ;
				float necky = tracked->neck.end.y ;
				
				if ((tracked->right_shoulder.found) && (tracked->left_shoulder.found)) {
					neckVector.x = tracked->neck.begin.x - tracked->neck.end.x;
					neckVector.y = tracked->neck.begin.y - tracked->neck.end.y;
					neckVector.z = tracked->neck.begin.z - tracked->neck.end.z;
					shoulderVector.x = tracked->right_shoulder.end.x - tracked->right_shoulder.begin.x;
					shoulderVector.y = tracked->right_shoulder.end.y - tracked->right_shoulder.begin.y;
					shoulderVector.z = tracked->right_shoulder.end.z - tracked->right_shoulder.begin.z;
					
					ofxVec3f booLocRaw = neckVector.cross(shoulderVector);
					ofxVec3f booLocNorm = booLocRaw.normalize();
					booLocationVector = booLocNorm.scale(200);
					
					boo.setAnchorPercent(0.5, 0.5);
					booAbsoluteLocationX = neck.end.x + booLocationVector.x;
					booAbsoluteLocationY = neck.end.y + booLocationVector.y + (10 * cos(angle));
					booDepth = neck.end.z + booLocationVector.z;
					
					cameraPixels = image.image_pixels;
					
					int scaledBooSize = 107*1300/booDepth;
					
					if ((booLocationVector.z > 0) && (booLocationVector.x < 0)) {
						boo.loadImage("boo.png");
						boo.resize(scaledBooSize, scaledBooSize);
						booPixels = boo.getPixels();
					}
					else if ((booLocationVector.z > 0) && (booLocationVector.x > 0)) {
						booFlip.loadImage("booFlip.png");
						booFlip.resize(scaledBooSize, scaledBooSize);
						booPixels = booFlip.getPixels();
						
					}
					else if ((booLocationVector.z < 0) && (booLocationVector.x < 0)) {
						booBack.loadImage("booBack.png");
						booBack.resize(scaledBooSize, scaledBooSize);
						booPixels = booBack.getPixels();
						booLaugh.play();
					}
					else {
						booBackFlip.loadImage("booBackFlip.png");
						booBackFlip.resize(scaledBooSize, scaledBooSize);
						booPixels = booBackFlip.getPixels();
					}
					
					for (int x = booAbsoluteLocationX; x < booAbsoluteLocationX + scaledBooSize; x ++) {
						for (int y = booAbsoluteLocationY; y < booAbsoluteLocationY + scaledBooSize; y ++) {
							int i = x + y * 640;
							int j = (x - booAbsoluteLocationX) + (y - booAbsoluteLocationY) * scaledBooSize;
							
							XnDepthPixel thisPixelDepth = theDepth[i];
							
							if((thisPixelDepth > booDepth) || (thisPixelDepth == 0)) { //draw boo first
								int r = booPixels[j * 4 + 0];
								int g = booPixels[j * 4 + 1];
								int bl = booPixels[j * 4 + 2];
								ofSetColor(r, g, bl, booPixels[j * 4 + 3]);
							}
							else { //draw camera first
								int r = cameraPixels[i * 3 + 0];
								int g = cameraPixels[i * 3 + 1];
								int bl = cameraPixels[i * 3 + 2];
								ofSetColor(r, g, bl);
							}
							ofRect(x, y, 1, 1);
						}
					}					
					
					info << i << " booVector: "
					<< booLocationVector.x << " " 
					<< booLocationVector.y << " " 
					<< booLocationVector.z << endl; 
					
				}
				
			}
			
		}
	}
	ofDrawBitmapString(info.str(), 10, 500);
	
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){
	if (key == 'f')
		ofSetFullscreen(true);
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){
	
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){
	
}

